@@ -117,8 +117,8 @@ struct _cairo_observer {
  *
  * A #cairo_hash_entry_t contains both a key and a value for
  * #cairo_hash_table_t. User-derived types for #cairo_hash_entry_t must
- * be type-compatible with this structure (eg. they must have an
- * unsigned long as the first parameter. The easiest way to get this
+ * be type-compatible with this structure (eg. they must have a
+ * uintptr_t as the first parameter. The easiest way to get this
  * is to use:
  *
  * 	typedef _my_entry {
@@ -147,7 +147,7 @@ struct _cairo_observer {
  * the entry need not be initialized if so desired.
  **/
 struct _cairo_hash_entry {
-    unsigned long hash;
+    uintptr_t hash;
 };
 
 struct _cairo_array {
@@ -194,6 +194,7 @@ struct _cairo_font_options {
     cairo_hint_style_t hint_style;
     cairo_hint_metrics_t hint_metrics;
     cairo_round_glyph_positions_t round_glyph_positions;
+    char *variations;
 };
 
 struct _cairo_glyph_text_info {
@@ -253,7 +254,8 @@ typedef enum _cairo_internal_surface_type {
     CAIRO_INTERNAL_SURFACE_TYPE_TEST_PAGINATED,
     CAIRO_INTERNAL_SURFACE_TYPE_TEST_WRAPPING,
     CAIRO_INTERNAL_SURFACE_TYPE_NULL,
-    CAIRO_INTERNAL_SURFACE_TYPE_TYPE3_GLYPH
+    CAIRO_INTERNAL_SURFACE_TYPE_TYPE3_GLYPH,
+    CAIRO_INTERNAL_SURFACE_TYPE_QUARTZ_SNAPSHOT
 } cairo_internal_surface_type_t;
 
 typedef enum _cairo_internal_device_type {
@@ -375,6 +377,8 @@ typedef struct _cairo_stroke_style {
     double		*dash;
     unsigned int	 num_dashes;
     double		 dash_offset;
+    cairo_bool_t	 is_hairline;
+    double      pre_hairline_line_width;
 } cairo_stroke_style_t;
 
 typedef struct _cairo_format_masks {
