@@ -56,9 +56,7 @@
  * enough to just add a new check to the function it complained about.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <assert.h>
 #include <limits.h>
@@ -68,9 +66,6 @@
 #if CAIRO_HAS_GL_SURFACE
 #include <cairo-gl.h>
 #endif
-#if CAIRO_HAS_OS2_SURFACE
-#include <cairo-os2.h>
-#endif
 #if CAIRO_HAS_PDF_SURFACE
 #include <cairo-pdf.h>
 #endif
@@ -1095,7 +1090,7 @@ static cairo_test_status_t
 test_cairo_surface_set_mime_data (cairo_surface_t *surface)
 {
     const char *mimetype = "text/x-uri";
-    const char *data = "http://www.cairographics.org";
+    const char *data = "https://www.cairographics.org";
     cairo_status_t status;
 
     status = cairo_surface_set_mime_data (surface,
